#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from gettext import gettext as _
import os
import sys
import argparse

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Gio
from gi.repository import GLib

from .utils import *
from .configuration import *
from .window.application import ApplicationWindow
from .window.about import AboutWindow

class Application(Gtk.Application):

    def __init__(self):
        Gtk.Application.__init__(self,
                                 application_id = APP_ID,
                                 flags = Gio.ApplicationFlags.HANDLES_COMMAND_LINE)

        GLib.set_application_name(APP_NAME)
        GLib.set_prgname(APP_COMMAND)

        #Gio.Application.get_default().set_value("test", "example")

        #print (Gio.Application.get_default().set_value("test"))

        self.window = None
        self.args = None


    """
    Gtk.Application startup handler
    """
    def do_startup(self):
        Gtk.Application.do_startup(self)

        self.build_menu()

    """
    Gtk.Application activate handler
    The starts the window if one does not exist and shows them all.
    """
    def do_activate(self):
        # Do command line
        if self.args.version:
            print(APP_VERSION)
            return

        if self.args.debug:
            print("Starting debug...")

        # Window already exists, return that
        if self.window:
            return

        # Else create a new application window.
        self.window = ApplicationWindow(self)

        # Connect quit event (normally already done but lets do it anyway)
        self.window.connect("delete_event", self.on_quit)

        # Show all the windows.
        # You MUST use .show_all() for Gtk.headerBar() to work!
        # .present() is a wrapper of .show() and .show() only shows the current window.
        self.window.show_all()

    """
    Gtk.Application command line handler
    Called if Gio.ApplicationFlags.HANDLES_COMMAND_LINE is set.
    """
    def do_command_line(self, args):
        Gtk.Application.do_command_line(self, args)

        # Start parsing
        parser = argparse.ArgumentParser(description=APP_DESCRIPTION,
                                         prog=APP_COMMAND)

        # Add debug option (-d/--debug)
        parser.add_argument('-d', '--debug',
                            action='store_true',
                            default=False,
                            dest="debug",
                            help=_("run in debug mode and show extra output")
                           )

        # Add version option (-v/--version)
        parser.add_argument('-v', '--version',
                            action='store_true',
                            default=False,
                            dest="version",
                            help=_("return the current version")
                           )

        # Store arguments so application can use them
        # We strip off the first argument (the file name)
        self.args = parser.parse_args(args.get_arguments()[1:])

        # Since we are calling do_command_line() we need to call do_activate()
        # For the program to continue
        self.do_activate()

        return 0

    """
    Build Gio Menu and add acclerator actions.
    """
    def build_menu(self):
        actions = [
            ('preferences', self.on_preferences),
            ('shortcuts', self.on_shortcuts),
            ('help', self.on_help),
            ('about', self.on_about),
            ('quit', self.on_quit),
        ]

        for action, callback in actions:
            simpleAction = Gio.SimpleAction.new(action, None)
            simpleAction.connect("activate", callback)
            self.add_action(simpleAction)

        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(APP_RESOURCES, "ui", "app-menu.ui"))
        self.menu = builder.get_object("app-menu")

        # Add app-menu to app
        self.set_app_menu(self.menu)


    """
    Callbacks for Gio.Menu
    """
    def on_preferences(self, action = None, param = None):
        preferences = PreferencesWindow(app = self)
        preferences.show()

    def on_help(self, action = None, param = None):
        Gtk.show_uri(None, APP_MANUAL_URL_OFFLINE, Gdk.CURRENT_TIME)

    def on_shortcuts(self, action = None, param = None):
        shortcuts = ShortcutsWindow(app = self)
        shortcuts.show()

    def on_about(self, action = None, param = None):
        about = AboutWindow(app = self)
        about.show()

    """
    Gtk.Application shutdown handler
    Do clean up before the application is closed, useful when you need
    to close a database connection or save work for the application.
    This is triggered when self.quit() is called.
    """
    def do_shutdown(self):
        Gtk.Application.do_shutdown(self)

    """
    Gtk.Application quit handler
    Quit the application when requested.
    """
    def on_quit(self, action = None, param = None):
        self.quit()

def main():
    app = Application()
    exit_status = app.run(sys.argv)
    sys.exit(exit_status)
